using System;
using System.ComponentModel;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;

namespace Dapfor.Net.Demo.Examples.DataBinding.HierarchicalDatabinding
{
    [Example("Hierarchical data binding", "Data binding")]
    public partial class HierarchicalDatabindingForm : UserControl
    {
        private readonly BindingList<Customer> _customers;
        private readonly Random _r = new Random();

        public HierarchicalDatabindingForm()
        {
            InitializeComponent();

            _customers = CreateData();

            grid1.DataSource = _customers;
            grid2.DataSource = _customers;
        }

        private static BindingList<Customer> CreateData()
        {
            BindingList<Customer> customers = new BindingList<Customer>();

            Customer customer = new Customer("Ann Adams", "Microsoft", "111-11-1111");
            customer.Orders.Add(new Order("111111", ".Net Grid", 2, 354.54));
            customer.Orders.Add(new Order("111112", ".Net Suite", 5, 395.21));
            customer.Orders.Add(new Order("111113", ".Net Chart", 1, 145.77));
            customers.Add(customer);

            customer = new Customer("Beth Baker", "Oracle", "222-22-2222");
            customer.Orders.Add(new Order("111120", ".Net Report", 2, 354.11));
            customer.Orders.Add(new Order("111121", ".Net Tab", 1, 185.73));
            customer.Orders.Add(new Order("111122", ".Net Office", 8, 132.15));
            customers.Add(customer);

            customer = new Customer("Cecil Carleton", "Google", "333-33-3333");
            customer.Orders.Add(new Order("111130", ".Net Security", 2, 654.41));
            customer.Orders.Add(new Order("111131", ".Net Lab", 4, 647.34));
            customer.Orders.Add(new Order("111132", ".Net Suite", 1, 451.65));
            customers.Add(customer);

            customer = new Customer("David Davis", "IBM", "444-44-4444");
            customer.Orders.Add(new Order("111141", ".Net Grid", 20, 354.54));
            customer.Orders.Add(new Order("111142", ".Net Suite", 5, 395.21));
            customer.Orders.Add(new Order("111143", ".Net Chart", 7, 145.77));
            customers.Add(customer);

            customer = new Customer("Ellen Edwards", "JPMorgan Chase & Co", "555-55-5555");
            customer.Orders.Add(new Order("111150", ".Net Report", 24, 354.11));
            customer.Orders.Add(new Order("111151", ".Net Tab", 10, 185.73));
            customer.Orders.Add(new Order("111152", ".Net Office", 18, 132.15));
            customers.Add(customer);

            return customers;
        }

        #region Private event handlers

        private void OnAddCustomerClick(object sender, EventArgs e)
        {
            _customers.Add(new Customer("customer #" + (_customers.Count + 1), "Company #" + _r.Next(1, 20), "000-00-0000"));
        }

        private void OnAddOrderClick(object sender, EventArgs e)
        {
            foreach (Customer customer in _customers)
            {
                customer.Orders.Add(new Order("Order #" + (customer.Orders.Count + 1), "Product #" + _r.Next(1, 20), _r.Next(1, 20), _r.Next(100, 600) + _r.NextDouble()));
            }
        }

        private void OnRemoveCustomerClick(object sender, EventArgs e)
        {
            if (_customers.Count > 0)
            {
                _customers.RemoveAt(_r.Next(0, _customers.Count));
            }
        }

        private void OnRemoveOrderClick(object sender, EventArgs e)
        {
            foreach (Customer customer in _customers)
            {
                if (customer.Orders.Count > 0)
                {
                    customer.Orders.RemoveAt(_r.Next(0, customer.Orders.Count));
                }
            }
        }

        #endregion
    }
}