using System;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Design;
using System.Windows.Forms.Design;

namespace Dapfor.Net.Demo.Examples.Features.CustomEditors
{
    /// <summary>
    /// Implementation of a custom editor
    /// </summary>
    public class CustomColorEditor : UITypeEditor
    {
        //We can resize the drop-down editor
        public override bool IsDropDownResizable
        {
            get { return true; }
        }

        public override bool GetPaintValueSupported(ITypeDescriptorContext context)
        {
            //Indicates, that the editor supports the value painting 
            return true;
        }

        public override void PaintValue(PaintValueEventArgs e)
        {
            //Paint the color
            using(Brush b = new SolidBrush((Color) e.Value))
            {
                e.Graphics.FillRectangle(b, e.Bounds);    
            }
        }

        public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value)
        {
            //Edit the cell. For more details about IWindowsFormsEditorService see MSDN 
            IWindowsFormsEditorService service = (IWindowsFormsEditorService) provider.GetService(typeof (IWindowsFormsEditorService));
            if (service != null)
            {
                //Create control
                using (CustomColorControl control = new CustomColorControl(service, (Color)value))
                {
                    //Start edit
                    service.DropDownControl(control);

                    //The edit id finished here, return a new color
                    value = control.EditingColor;
                }
            }
            return value;
        }

        public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context)
        {
            //Drop-down style
            return UITypeEditorEditStyle.DropDown;
        }
    }
}