using System;
using System.Drawing;
using System.Windows.Forms;
using Dapfor.Net.Demo.Properties;
using Dapfor.Net.Editors;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.Features.CustomEditors
{
    public class PlusMinusEditor : UITypeEditorEx
    {
        public override bool GetPaintCellSupported()
        {
            return true;
        }

        public override void PaintCell(PaintCellEventArgs e)
        {
            string text = e.Text;
            
            //Do basic painting without text
            e.Text = string.Empty;
            e.PaintAll();
            e.Handled = true;

            //Compute minus button bounds
            Rectangle minusRect = e.Cell.VirtualBounds;
            minusRect.Width = Resources.error.Width;

            //Compute plus button bounds
            Rectangle plusRect = e.Cell.VirtualBounds;
            plusRect.X = e.Cell.VirtualBounds.Right - Resources.error.Width;
            plusRect.Width = Resources.error.Width;

            //Compute text bounds
            Rectangle textRect = e.Cell.VirtualBounds;
            textRect.X = minusRect.Right;
            textRect.Width = Math.Max(0, plusRect.Left - minusRect.Right);

            //Draw text
            e.PaintText(text, textRect);

            //Draw buttons
            e.Graphics.DrawImage(Resources.error, minusRect);
            e.Graphics.DrawImage(Resources.add, plusRect);
        }

        public override StopEditReason EditCell(IGridEditorService service, Cell cell, StartEditReason reason)
        {
            StopEditReason stopReason = StopEditReason.Undefined;
            if (Equals(StartEditReason.LButtonClick, reason) && cell.Column != null && cell.Row != null)
            {
                int delta = 0;
                Point pt = cell.Row.Grid.PointToClient(Cursor.Position);
                
                //Plus rectangle
                Rectangle rect = cell.VirtualBounds;
                rect.X = cell.VirtualBounds.Right - Resources.error.Width;
                rect.Width = Resources.add.Width;
                if (rect.Contains(pt))
                {
                    delta = 1;
                }
                else
                {
                    //Minus rectangle
                    rect.X = cell.VirtualBounds.Left;
                    rect.Width = Resources.error.Width;
                    if (rect.Contains(pt))
                    {
                        delta = -1;    
                    }
                }

                if(delta != 0)
                {
                    cell.Value = (int)cell.Value + delta;
                    stopReason = StopEditReason.UserStop;
                }
            }
            return stopReason;
        }
    }
}