using System;
using System.Text;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.Features.DragAndDrop
{
    [Example("Drag & drop", "Features")]
    public partial class DragAndDropForm : UserControl
    {
        public DragAndDropForm()
        {
            InitializeComponent();

            //Populate the first grid
            for (int i = 0; i < 100; ++i)
            {
                grid1.Rows.Add(new DataObject("Item " + i, i, i%5));
            }

            //Populate the second grid
            for (int i = 0; i < 100; ++i)
            {
                grid2.Rows.Add(new DataObject("Item " + i, i, i%5));
            }
        }

        private void grid_DragBegin(object sender, DragContentBeginEventArgs e)
        {
            if ((e.Source == grid1 && cbLeftExport.Checked) || (e.Source == grid2 && cbRightExport.Checked))
            {
                StringBuilder sb = new StringBuilder();
                foreach (Row row in e.Content)
                {
                    if (sb.Length > 0) sb.AppendLine();
                    StringBuilder sbRow = new StringBuilder();
                    foreach (Column column in row.Header)
                    {
                        if (sbRow.Length > 0) sbRow.Append("\t");
                        sbRow.Append(row[column.Id].Text);
                    }
                    sb.Append(sbRow);
                }
                e.Data = sb.ToString();
            }
        }

        private void grid_DragOverContent(object sender, DragContentEventArgs e)
        {
            if ((e.TargetGrid == grid1 && cbLeftImport.Checked) || (e.TargetGrid == grid2 && cbRightImport.Checked))
            {
                //The content is not specified. The data can be found in e.Data (IDataObject)
                if (e.Content == null)
                {
                    e.Effect = DragDropEffects.Copy;    
                }
            }
        }

        private void grid_DragDropContent(object sender, DragDropContentEventArgs e)
        {
            if (e.Content == null)
            {
                string data = e.Data.GetData(typeof (string)) as string;
                if (!string.IsNullOrEmpty(data))
                {
                    string[] lines = data.Split(new string[] {Environment.NewLine}, StringSplitOptions.RemoveEmptyEntries);
                    foreach (string line in lines)
                    {
                        string[] values = line.Split(new string[] {"\t"}, StringSplitOptions.None);

                        //Add an array of IEnumerable to the grid. Note, here you can create your own object and insert to the grid 
                        //instead of string values
                        e.DoCopy(values);
                    }

                    e.Handled = true;
                }
            }
        }

        #region Event handlers

        private void cbLeftDrag_CheckedChanged(object sender, EventArgs e)
        {
            grid1.DragAndDrop.Enabled = cbLeftDrag.Checked;
        }

        private void cbLeftDrop_CheckedChanged(object sender, EventArgs e)
        {
            grid1.AllowDrop = cbLeftDrop.Checked;
        }

        private void rbLeftGrid_ModeChanged(object sender, EventArgs e)
        {
            GridDropMode mode = GridDropMode.Flat;
            if (rbLeftFlatAndHierarchy.Checked)
            {
                mode = GridDropMode.FlatAndAsChild;
            }
            else if (rbLeftHierarchy.Checked)
            {
                mode = GridDropMode.AsChild;
            }
            grid1.DragAndDrop.DropMode = mode;
        }

        private void cbRightDrag_CheckedChanged(object sender, EventArgs e)
        {
            grid2.DragAndDrop.Enabled = cbRightDrag.Checked;
        }

        private void cbRightDrop_CheckedChanged(object sender, EventArgs e)
        {
            grid2.AllowDrop = cbRightDrop.Checked;
        }

        private void rbRightGrid_ModeChanged(object sender, EventArgs e)
        {
            GridDropMode mode = GridDropMode.Flat;
            if (rbRightFlatAndHierarchy.Checked)
            {
                mode = GridDropMode.FlatAndAsChild;
            }
            else if (rbRightHierarchy.Checked)
            {
                mode = GridDropMode.AsChild;
            }
            grid2.DragAndDrop.DropMode = mode;
        }

        #endregion
    }
}