using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Design;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Editors;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.Features.EditorsInDockedRows
{
    [Example("Editors in docked rows", "Features")]
    public partial class DockedRowsForm : UserControl
    {
        public DockedRowsForm()
        {
            InitializeComponent();


            //Populate the grid
            Random r = new Random();
            for(int i = 0; i < 10; ++i)
            {
                Row row = grid.Rows.Add(new object[] { r.Next(10000), r.Next(10000), r.Next(10000), r.Next(10000), r.Next(10000) });
                for(int j = 0; j < 5; ++j)
                {
                    row.Add(new object[] { r.Next(10000), "Product " + r.Next(1, 10), r.Next(10000), r.Next(1000)}); 
                }

                Row dockedTop = row.Add(new object[] {row.Children[r.Next(row.Children.Count - 1)][0].Value, 
                                                      row.Children[r.Next(row.Children.Count - 1)][1].Value, 
                                                      r.Next(10000), 
                                                      row.Children[r.Next(row.Children.Count - 1)][3].Value,  });
                dockedTop.Dock = RowDockStyle.Top;

                Row dockedBottom = row.Add(new object());
                dockedBottom.Dock = RowDockStyle.Bottom;
            }

            grid.Rows[0].Dock = RowDockStyle.Top;

            //Set sort. 
            grid.Headers[0][1].SortDirection = SortDirection.Ascending;
            grid.Headers[1][3].SortDirection = SortDirection.Ascending;
            grid.Headers[1][2].SortDirection = SortDirection.Descending;

            grid.Nodes[3].Expanded = true;
            grid.Nodes[8].Expanded = true;
        }

        void OnPaintRow(object sender, PaintRowEventArgs e)
        {
            if (e.Row.Dock == RowDockStyle.Top)
            {
                e.PaintAll();
                e.Handled = true;

                using (StringFormat sf = new StringFormat())
                {
                    sf.Alignment = StringAlignment.Center;
                    sf.LineAlignment = StringAlignment.Center;
                    Rectangle rect = e.Row.Bounds;
                    rect.X = e.Row[3].VisibleBounds.Right;
                    rect.Width = e.Row.Bounds.Right - rect.X;
                    e.Graphics.DrawString("This row is docked to the top", e.Font, SystemBrushes.InfoText, rect, sf);
                }
            }

            if(e.Row.Dock == RowDockStyle.Bottom)
            {
                e.Appearance.BackColor = Color.FromArgb(211, 243, 209);
                e.Appearance.GradientEnabled = true;
                e.Appearance.GradientEndBackColor = Color.FromArgb(181, 241, 185);
                e.Appearance.GradientDirection = GradientDirection.Vertical;

                e.Parts &= e.Parts ^ PaintPart.Text;
                e.PaintAll();
                e.Handled = true;
                
                using(StringFormat sf = new StringFormat())
                {
                    sf.Alignment = StringAlignment.Center;
                    sf.LineAlignment = StringAlignment.Center;
                    e.Graphics.DrawString("This row is docked to the bottom", e.Font, SystemBrushes.InfoText, e.Row.Bounds, sf);    
                }
            }
        }

        private void OnCellEndEdit(object sender, GridCellEndEditEventArgs e)
        {
            e.Grid.Invalidate(false);
        }

        void OnCellEditor(object sender, GridCellEditableEventArgs e)
        {
            UITypeEditor editor = e.Editor;
            if (e.Cell.Row.Dock == RowDockStyle.Top && e.Cell.Row.Level == 1)
            {
                //Dropdown menu with integer values
                if (e.Cell.Column.Index == 0)
                {
                    List<int> collection = new List<int>();
                    foreach (Row child in e.Cell.Row.Parent.Children)
                    {
                        if (child.Dock == RowDockStyle.None)
                        {
                            int value = (int)child[e.Cell.Column.Id].Value;
                            if (!collection.Contains(value))
                            {
                                collection.Add(value);
                            }
                        }
                    }

                    editor = new EnumerableEditor(collection);
                }
                //Dropdown menu with string values
                if (e.Cell.Column.Index == 1)
                {
                    List<string> collection = new List<string>();
                    foreach (Row child in e.Cell.Row.Parent.Children)
                    {
                        if (child.Dock == RowDockStyle.None)
                        {
                            string value = (string)child[e.Cell.Column.Id].Value;
                            if (!collection.Contains(value))
                            {
                                collection.Add(value);
                            }
                        }
                    }

                    editor = new EnumerableEditor(collection, true);
                }

                    //Trackbar menu
                else if (e.Cell.Column.Index == 3)
                {
                    editor = new TrackBarEditor(0, 1000);
                }
            }
            e.Editor = editor;
            e.Editable = editor != null;
        }

        private void OnCustomPaint(object sender, PaintCellEventArgs e)
        {
            //Row docked to the bottom
            if (e.Cell.Row != null && e.Cell.Row.Dock == RowDockStyle.Bottom)
            {
                e.Parts &= e.Parts ^ PaintPart.Text;
            }
            //Row docked to the top
            if (e.Cell.Row != null && e.Cell.Row.Dock == RowDockStyle.Top)
            {
                e.Appearance.BackColor = Color.FromArgb(254, 206, 206);
                e.Appearance.GradientEnabled = true;
                e.Appearance.GradientEndBackColor = Color.FromArgb(254, 176, 176);
                e.Appearance.GradientDirection = GradientDirection.Vertical;
            }
            //Non-rows on the first hierarchical level
            if (e.Cell.Row != null && e.Cell.Column != null && e.Cell.Row.Dock == RowDockStyle.None && e.Cell.Row.Level == 1)
            {
                //Get a filter's value
                object filterValue = e.Cell.Row.Parent.Children[0][e.Cell.Column.Id].Value;
                bool match = false;


                //Update color in the painting cell
                if (Equals(filterValue, e.Cell.Value) && e.Cell.Column.Index == 0)
                {
                    match = true;
                }
                if (Equals(filterValue, e.Cell.Value) && e.Cell.Column.Index == 1)
                {
                    match = true;
                }
                if ( e.Cell.Column.Index == 3)
                {
                    if((int)filterValue > (int)e.Cell.Value)
                    {
                        match = true;
                    }
                }

                //row matches to filter
                if(match)
                {
                    e.Appearance.BackColor = Color.FromArgb(211, 209, 243);
                    e.Appearance.GradientEnabled = true;
                    e.Appearance.GradientEndBackColor = Color.FromArgb(181, 185, 241);
                    e.Appearance.GradientDirection = GradientDirection.Vertical;
                }
            }
        }
    }
}