using System;
using System.Drawing;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.Features.Summaries
{
    [Example("Summary rows", "Features")]
    public partial class SummariesForm : UserControl
    {
        readonly Font _summaryFont = new Font("Arial", 8, FontStyle.Bold | FontStyle.Italic);

        public SummariesForm()
        {
            InitializeComponent();

            Random r = new Random();
            for (int i = 0; i < 5; ++i)
            {
                Row row = grid.Rows.Add(new string[] {"Object " + i});
                for (int j = 0; j < r.Next(10, 30); j++)
                {
                    row.Add(new int[] {r.Next(1000), r.Next(1000), r.Next(1000)});
                }

                Row sumTopRow = row.Add(new object());
                sumTopRow.Dock = RowDockStyle.Top;

                Row sumBottomRow = row.Add(new object());
                sumBottomRow.Dock = RowDockStyle.Bottom;
                row.Expanded = true;
            }

            grid.Headers[0][0].SortDirection = SortDirection.Ascending;
            grid.Headers[1][0].SortDirection = SortDirection.Ascending;
            grid.Headers[1][2].SortDirection = SortDirection.Descending;
        }

        private void grid_PaintRow(object sender, PaintRowEventArgs e)
        {
            if (Equals(RowDockStyle.Bottom, e.Row.Dock))
            {
                //Do default painting routines
                e.Appearance.BackColor = Color.FromArgb(226, 237, 252);
                e.Appearance.GradientEndBackColor = Color.FromArgb(198, 218, 249);
                e.Appearance.GradientEnabled = true;
                e.Appearance.GradientDirection = GradientDirection.Vertical;
                e.PaintAll();
                e.Handled = true;

                string s = string.Format("Sum1 = {0}, Sum2 = {1}, Sum3 = {2}",
                    GetSummary(e.Row.Parent, grid.Headers[1]["Id1"]),
                    GetSummary(e.Row.Parent, grid.Headers[1]["Id2"]),
                    GetSummary(e.Row.Parent, grid.Headers[1]["Id3"]));

                e.Graphics.DrawString(s, _summaryFont, Brushes.Black, e.Bounds);
            }
            else if (Equals(RowDockStyle.Top, e.Row.Dock))
            {
                e.Appearance.BackColor = Color.FromArgb(254, 243, 224);
                e.Appearance.GradientEndBackColor = Color.FromArgb(252, 227, 182);
                e.Appearance.GradientEnabled = true;
                e.Appearance.GradientDirection = GradientDirection.Vertical;
            }
        }

        private void grid_PaintCell(object sender, PaintCellEventArgs e)
        {
            if (e.Cell.Column != null && Equals(RowDockStyle.Top, e.Cell.Row.Dock))
            {
                e.Text = string.Format("{0}", GetSummary(e.Cell.Row.Parent, e.Cell.Column));
                e.Font = _summaryFont;
            }
        }

        static int GetSummary(Row parentRow, Column column)
        {
            int sum = 0;
            foreach (Row child in parentRow.Children)
            {
                //Don't count sum rows.
                if (Equals(RowDockStyle.None, child.Dock))
                {
                    sum += (int)child[column.Id].Value;
                }
            }
            return sum;
        }
    }
}