using System;
using System.Drawing;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.Features.SummariesInColumns
{
    [Example("Summaries in columns", "Features")]
    public partial class SummariesInColumnsForm : UserControl
    {
        readonly Font _summaryFont = new Font("Arial", 8, FontStyle.Bold | FontStyle.Italic);

        public SummariesInColumnsForm()
        {
            InitializeComponent();

            Random r = new Random();
            for (int i = 0; i < 5; ++i)
            {
                Row row = grid.Rows.Add(new string[] { "Object " + i });
                for (int j = 0; j < r.Next(10, 30); j++)
                {
                    row.Add(new int[] { r.Next(1000), r.Next(1000), r.Next(1000) });
                }

                row.Expanded = true;
            }
        }

        private void grid_PaintColumnCaption(object sender, PaintColumnCaptionEventArgs e)
        {
            if (e.Column.Header.Level > 0)
            {
                //Do default drawing
                e.PaintAll();
                e.Handled = true;

                //Custom drawing area
                Rectangle rc = e.VirtualBounds;
                rc.Y += rc.Height / 2;
                rc.Height /= 2;


                //Get a row below the header
                Row row = grid.HitTests.RowTest(new Point(e.VisibleBounds.X, e.VisibleBounds.Bottom));

                //Get its parent
                Row parentRow = row != null ? row.Parent : null;

                //Compute summary
                if (parentRow != null)
                {
                    int sum1 = 0;
                    foreach (Row child in parentRow.Children)
                    {
                        //Don't count the sum row.
                        if (Equals(RowDockStyle.None, child.Dock))
                        {
                            sum1 += (int)child[e.Column.Id].Value;
                        }
                    }

                    using (StringFormat sf = new StringFormat())
                    {
                        sf.Alignment = StringAlignment.Center;
                        sf.FormatFlags = StringFormatFlags.NoWrap;
                        e.Graphics.DrawString(string.Format("Sum = {0}", sum1), _summaryFont, Brushes.DarkGreen, rc, sf);
                    }
                }
            }

        }
    }
}