using System;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.Globalization;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Formats;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.GridBasics.DataSourceFiltering
{
    [Example("Data source filtering and grouping", ".Net Grid basics")]
    public partial class DataSourceFilteringForm : UserControl
    {
        private readonly DataSet _dataSet;
        private double _filterMaxVal;
        private double _filterMinVal;


        public DataSourceFilteringForm()
        {
            InitializeComponent();

            //Initialize controls
            lbTotalRows.Text = string.Empty;
            lbVisibleRows.Text = string.Empty;
            tbMinValue.Text = "20";
            tbMaxValue.Text = "22";
            
            //Set a filter
            grid.Filter = new Filter(IsRowFiltered);

           //Create dataset
            _dataSet = CreateDataSet();

            //Attach datasource
            grid.Headers.AutoGenerate = true;

            BindingSource source = new BindingSource();
            source.DataMember = "Orders";
            source.DataSource = _dataSet;


            grid.DataSource = source;

            //Initialize columns
            grid.Headers[0]["orderId"].Editable = false;
            grid.Headers[0]["orderId"].Groupable = false;
            grid.Headers[0]["orderId"].Width = 100;
            
            grid.Headers[0]["product"].Editor = new ProductEditor(_dataSet.Tables["Orders"].Rows);
            grid.Headers[0]["product"].SortDirection = SortDirection.Ascending;
            grid.Headers[0]["product"].Grouped = true;
            grid.Headers[0]["product"].Width = 100;
            grid.Headers[0]["product"].Visible = false;
            
            grid.Headers[0]["amount"].Format = new DoubleFormat(2, false, false);
            grid.Headers[0]["amount"].Appearance.CellTextAlignment = ContentAlignment.MiddleRight;
            grid.Headers[0]["amount"].Groupable = false;
            grid.Headers[0]["amount"].Width = 100;
            
            grid.Headers[0]["quantity"].Appearance.CellTextAlignment = ContentAlignment.MiddleRight;
            grid.Headers[0]["quantity"].Format = new LongFormat();
            grid.Headers[0]["quantity"].Groupable = false;
            grid.Headers[0]["quantity"].Width = 100;

            grid.Headers[0].GroupingEnabled = true;
        }

        public static DataSet CreateDataSet()
        {
            //Create some products
            IList<string> products = new List<string>();
            for (int i = 1; i < 10; ++i)
            {
                products.Add(string.Format("Product {0}", i));
            }

            // create a DataSet with one table, two columns
            DataSet dataSet = new DataSet();

            // create Customer table
            DataTable table = new DataTable("Orders");
            dataSet.Tables.Add(table);
            table.Columns.Add("orderId", typeof (int)).AutoIncrement = true;
            table.Columns.Add("product", typeof (string));
            table.Columns.Add("amount", typeof (double));
            table.Columns.Add("quantity", typeof (long));
            table.PrimaryKey = new DataColumn[] {table.Columns["orderId"]};

            //populate the table
            Random random = new Random();
            for (int orderId = 1; orderId <= 1000; orderId++)
            {
                // add customer record
                table.Rows.Add(new object[] {orderId, products[random.Next()%products.Count], (double) random.Next(20, 100)/random.Next(1, 10), (long) random.Next(1, 10)});
            }

            return dataSet;
        }

        private void OnUpdateFilter(object sender, EventArgs e)
        {
            tbMinValue.Enabled = cbMinValue.Checked;
            tbMaxValue.Enabled = cbMaxValue.Checked;

            double.TryParse(tbMinValue.Text, NumberStyles.Number, CultureInfo.InvariantCulture, out _filterMinVal);
            double.TryParse(tbMaxValue.Text, NumberStyles.Number, CultureInfo.InvariantCulture, out _filterMaxVal);
            grid.FilterRefresh();
        }

        private bool IsRowFiltered(Row row)
        {
            bool filtered = false;

            //The value may be null or DBNull
            if (row["amount"].Value is double)
            {
                double amount = (double) row["amount"].Value;
                filtered = (cbMinValue.Checked && amount < _filterMinVal) || (cbMaxValue.Checked && amount > _filterMaxVal);
            }
            return filtered;
        }

        private void OnGridRowUpdate(object sender, GridRowEventArgs e)
        {
            lbTotalRows.Text = _dataSet.Tables["Orders"].Rows.Count.ToString();
            lbVisibleRows.Text = grid.Rows.Count.ToString();
        }

        private void OnGridFocusedRowChanged(object sender, FocusedRowEventArgs e)
        {
            lbTotalRows.Text = _dataSet.Tables["Orders"].Rows.Count.ToString();
        }
    }
}