using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Demo.Properties;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.GridBasics.Selection
{
    [Example("Semi-transparent selection", ".Net Grid basics")]
    public partial class SelectionForm : UserControl
    {
        private readonly List<DataObject> _dataObjects = new List<DataObject>();
        private readonly Timer _timer = new Timer();
        private readonly Random _random = new Random();

        public SelectionForm()
        {
            InitializeComponent();

            //Selection colors
            clrSelection.Color = Color.FromArgb(255, grid.Selection.Color);
            tbSelection.Value = grid.Selection.Color.A;
            clrLasso.Color = Color.FromArgb(255, grid.Selection.LassoSelectionColor);
            tbLasso.Value = grid.Selection.LassoSelectionColor.A;

            CreateData();

            //Populate the grid with data
            foreach (DataObject dataObject in _dataObjects)
            {
                grid.Rows.Add(dataObject);
            }

            _timer.Interval = 100;
            _timer.Tick += OnTimerTick;
            _timer.Start();

            //Select some rows
            for(int i = 3; i < 8; ++i)
            {
                grid.Rows[i].Selected = true;
            }
            
            for (int i = 12; i < 24; ++i)
            {
                grid.Rows[i].Selected = true;
            }
        }

        private void CreateData()
        {
            _dataObjects.Add(new DataObject("ACCOR", 22.96, 546));
            _dataObjects.Add(new DataObject("AIR LIQUIDE", 84.99, 54));
            _dataObjects.Add(new DataObject("ALCATEL-LUCENT", 2.087, 1032));
            _dataObjects.Add(new DataObject("ALSTOM", 36.135, 1000));
            _dataObjects.Add(new DataObject("ARCELORMITTAL", 22.81, 3645));
            _dataObjects.Add(new DataObject("AXA", 12.695, 6532));
            _dataObjects.Add(new DataObject("BNP PARIBAS ACT.A", 46.835, 650));
            _dataObjects.Add(new DataObject("BOUYGUES", 30.785, 65));
            _dataObjects.Add(new DataObject("CAP GEMINI", 36.275, 694));
            _dataObjects.Add(new DataObject("CARREFOUR", 34.86, 368));
            _dataObjects.Add(new DataObject("CREDIT AGRICOLE", 8.872, 3540));
            _dataObjects.Add(new DataObject("DANONE", 45.365, 93));
            _dataObjects.Add(new DataObject("DEXIA", 3.11, 1360));
            _dataObjects.Add(new DataObject("EADS", 16.515, 1035));
            _dataObjects.Add(new DataObject("EDF", 30.445, 650));
            _dataObjects.Add(new DataObject("ESSILOR INTL.", 49.705, 469));
            _dataObjects.Add(new DataObject("FRANCE TELECOM", 14.85, 65));
            _dataObjects.Add(new DataObject("GDF SUEZ", 24.66, 87));
            _dataObjects.Add(new DataObject("L'OREAL", 83.04, 981));
            _dataObjects.Add(new DataObject("LAFARGE", 40.355, 64));
            _dataObjects.Add(new DataObject("LAGARDERE S.C.A.", 25.735, 94));
            _dataObjects.Add(new DataObject("LVMH", 89.03, 84));
            _dataObjects.Add(new DataObject("MICHELIN", 58.30, 14));
            _dataObjects.Add(new DataObject("PERNOD RICARD", 63.33, 5));
            _dataObjects.Add(new DataObject("PEUGEOT", 23.435, 47));
            _dataObjects.Add(new DataObject("PPR", 98.72, 36));
            _dataObjects.Add(new DataObject("RENAULT", 33.42, 41));
            _dataObjects.Add(new DataObject("SAINT GOBAIN", 31.455, 32));
            _dataObjects.Add(new DataObject("SANOFI-AVENTIS", 48.25, 12));
            _dataObjects.Add(new DataObject("SCHNEIDER ELECTRIC", 84.94, 43));
            _dataObjects.Add(new DataObject("SOCIETE GENERALE", 35.97, 73));
            _dataObjects.Add(new DataObject("STMICROELECTRONICS", 6.649, 184));
            _dataObjects.Add(new DataObject("SUEZ ENVIRONNEMENT", 13.82, 73));
            _dataObjects.Add(new DataObject("TECHNIP", 47.72, 6));
            _dataObjects.Add(new DataObject("TOTAL", 38.09, 87));
            _dataObjects.Add(new DataObject("UNIBAIL-RODAMCO", 135.55, 13));
            _dataObjects.Add(new DataObject("VALLOUREC", 75.91, 64));
            _dataObjects.Add(new DataObject("VEOLIA ENVIRON.", 19.42, 9));
            _dataObjects.Add(new DataObject("VINCI", 35.005, 16));
            _dataObjects.Add(new DataObject("VIVENDI", 17.285, 21));
        }

        #region Event handlers

        void OnTimerTick(object sender, EventArgs e)
        {
            DataObject data = _dataObjects[_random.Next(_dataObjects.Count)];
            Row row = grid.DataObjects.FindFirstRow(data);
            
            
            if(row != null)
            {
                //Bid and ask price & quntities
                if (_random.Next() % 2 == 0)
                {
                    double prevValue = data.Bid;
                    data.Bid = data.Last - data.Last * (1f / _random.Next(500, 1000));
                    data.BidQty = _random.Next(1, 50);
                    row[DataObject.FieldBid].Highlight(TimeSpan.FromSeconds(1), data.Bid > prevValue ? Color.Green : Color.Red);
                    row[DataObject.FieldBidQty].Highlight(TimeSpan.FromSeconds(1), data.Bid > prevValue ? Color.Green : Color.Red);
                }
                else
                {
                    double prevValue = data.Ask;
                    data.Ask = data.Last + data.Last * (1f / _random.Next(500, 1000));
                    data.AskQty = _random.Next(1, 50);
                    row[DataObject.FieldAsk].Highlight(TimeSpan.FromSeconds(1), data.Ask > prevValue ? Color.Green : Color.Red);
                    row[DataObject.FieldAskQty].Highlight(TimeSpan.FromSeconds(1), data.Ask > prevValue ? Color.Green : Color.Red);
                }
            }

            data = _dataObjects[_random.Next(_dataObjects.Count)];
            row = grid.DataObjects.FindFirstRow(data);

            if(row != null)
            {
                //Last execution price
                double delta = data.Last * (1f / _random.Next(500, 1000));
                delta = _random.Next() % 2 == 0 ? delta : -delta;

                data.Last = Math.Max(data.Last + delta, data.Open / 2);
                data.LastQty = _random.Next(1, 50);
                row[DataObject.FieldLast].Highlight(TimeSpan.FromSeconds(1), delta > 0 ? Color.Green : Color.Red);
                row[DataObject.FieldChange].Highlight(TimeSpan.FromSeconds(1), delta > 0 ? Color.Green : Color.Red);
            }

            if (_random.Next() % 10 == 0)
            {
                //Update state
                data = _dataObjects[_random.Next(_dataObjects.Count)];
                int v = _random.Next(100);
                if (v > 5)
                {
                    data.State = State.Ok;
                }
                else data.State = v > 2 ? State.PartialOk : State.Nok;
            }
        }

        private void OnGridPaintCell(object sender, PaintCellEventArgs e)
        {
            DataObject dataObject = e.Cell.Row.DataObject as DataObject;
            if (e.Cell.Column != null && dataObject != null)
            {
                switch (e.Cell.Column.Id)
                {
                    case DataObject.FieldHistory:
                        e.Parts &= e.Parts ^ PaintPart.Highlighting;
                        e.PaintBackground();
                        e.Parts &= e.Parts ^ PaintPart.Background;
                        DrawHistory(dataObject.History, e.Cell.VirtualBounds, e.Graphics);
                        break;

                    case DataObject.FieldState:
                        switch (dataObject.State)
                        {
                            case State.Ok:
                                e.Image = Resources.bullet_ball_glass_green;
                                break;
                            case State.Nok:
                                e.Image = Resources.bullet_ball_glass_red;
                                break;
                            case State.PartialOk:
                                e.Image = Resources.bullet_ball_glass_yellow;
                                break;
                        }
                        e.ImageSettings.Alignment = ContentAlignment.MiddleCenter;
                        e.Parts &= e.Parts ^ PaintPart.Text;
                        break;
                }
            }
        }

        private static void DrawHistory(IList<double> list, Rectangle bounds, Graphics graphics)
        {
            if (list != null && list.Count > 0 && bounds.Height > 0)
            {
                double min = 0;
                double max = 0;
                foreach (double d in list)
                {
                    min = min > 0 ? Math.Min(min, d) : d;
                    max = Math.Max(max, d);
                }
                double vRatio = (max - min) / bounds.Height;

                List<Point> points = new List<Point>();
                for (int i = 0; i < list.Count; ++i)
                {
                    double v = list[i];
                    int dy = (vRatio > 0 ? (int)((v - min) / vRatio) - 1 : bounds.Height / 2);
                    dy = Math.Max(0, dy);
                    int y = bounds.Y + dy;
                    int x = bounds.Right - ((i + 1) * bounds.Width / DataObject.MaxHistoryLength);
                    if (points.Count == 0)
                    {
                        points.Add(new Point(bounds.Right, y));
                    }
                    points.Add(new Point(x, y));
                }
                using (Pen p = new Pen(Color.Blue))
                {
                    graphics.DrawLines(p, points.ToArray());
                }
            }
        }

        private void OnUpdateSelection(object sender, EventArgs e)
        {
            rbSingle.Enabled = cbEnabled.Checked;
            rbMultiple.Enabled = cbEnabled.Checked;
            cbLasso.Enabled = rbMultiple.Checked && cbEnabled.Checked;
            grid.Selection.Enabled = cbEnabled.Checked;
            grid.Selection.MultipleEnabled = rbMultiple.Checked;
        }

        private void OnLassoSelectionUpdate(object sender, EventArgs e)
        {
            grid.Selection.LassoSelectionEnabled = cbLasso.Checked;
        }

        private void OnUpdateSelectionColor(object sender, EventArgs e)
        {
            grid.Selection.Color = Color.FromArgb(tbSelection.Value, clrSelection.Color);
        }
        
        private void OnUpdateLassoColor(object sender, EventArgs e)
        {
            grid.Selection.LassoSelectionColor = Color.FromArgb(tbLasso.Value, clrLasso.Color);
        }

        #endregion
    }
}