using System;
using System.IO;
using Dapfor.Net.Data;
using Dapfor.Net.Formats;

namespace Dapfor.Net.Demo.Examples.Miscellaneous.FileExplorer
{
    /// <summary>
    /// The objects of this class represents a file in the file system
    /// </summary>
    public sealed class FileObject : IExplorable
    {
        private readonly FileAttributes _attributes;
        private readonly IExplorable _baseExplorable;
        private readonly DateTime _creationDate;
        private readonly string _name;
        private readonly long _size;

        public FileObject(string name, DateTime creationDate, long size, FileAttributes attributes, IExplorable baseExplorable)
        {
            _name = name;
            _creationDate = creationDate;
            _size = size;
            _attributes = attributes;
            _baseExplorable = baseExplorable;
        }

        [Field("Name")]
        public string FileName
        {
            get { return _name; }
        }

        [Field("Date")]
        public DateTime CreationDate
        {
            get { return _creationDate; }
        }

        [Field("Attributes")]
        public FileAttributes Attributes
        {
            get { return _attributes; }
        }

        [Field("Size")]
        [Format(typeof (SizeCustomFormat))]
        public long Size
        {
            get { return _size; }
        }

        #region IExplorable Members

        public string FullName
        {
            get { return _baseExplorable.FullName + _name; }
        }

        public bool Expandable
        {
            get { return false; }
        }

        #endregion
    }
}