using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Theming;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.Miscellaneous.Themes
{
    [Example("Themes", "Miscellaneous")]
    public partial class ThemesForm : UserControl
    {
        public ThemesForm()
        {
            InitializeComponent();

            ArrayList themes = new ArrayList(Enum.GetValues(typeof (KnownTheme)));
            themes.Sort(new Comparer());
            themes.Insert(0, "<None>");
            cbTheme.DataSource = themes;

            //Populate with random data
            grid.DataSource = GetData();

            grid.Headers[0][0].Grouped = true;
            grid.Headers[0][0].Visible = false;
            grid.Rows.ExpandAll();
            grid.Rows[5].Focused = true;
            for (int i = 5; i < 15; ++i)
            {
                grid.Rows[i].Selected = true;
            }
        }

        private void cbTheme_SelectedIndexChanged(object sender, EventArgs e)
        {
            Theme theme = null;
            if (cbTheme.SelectedIndex > 0)
            {
                KnownTheme knownTheme = (KnownTheme) Enum.Parse(typeof (KnownTheme), cbTheme.SelectedItem.ToString(), true);
                theme = Theme.FromKnownTheme(knownTheme);
            }
            grid.Theme = theme;
        }

        private IBindingList GetData()
        {
            Dictionary<string, IList<string>> manufacturers = new Dictionary<string, IList<string>>();
            Random random = new Random();

            List<string> models = new List<string>();
            models.Add("Golf");
            models.Add("Polo");
            models.Add("Passat");
            models.Add("Touran");
            manufacturers.Add("Volkswagen", models);

            models = new List<string>();
            models.Add("X1");
            models.Add("X3");
            models.Add("X5");
            models.Add("M3 Sedan");
            models.Add("Z4 Roadster");
            manufacturers.Add("BMW", models);

            models = new List<string>();
            models.Add("A-Class");
            models.Add("B-Class");
            models.Add("SLK");
            models.Add("CLS");
            manufacturers.Add("Mercedes", models);


            models = new List<string>();
            models.Add("Scenic");
            models.Add("Duster");
            models.Add("Kangoo");
            models.Add("Clio");
            models.Add("Megane");
            manufacturers.Add("Renault", models);

            models = new List<string>();
            models.Add("308");
            models.Add("407");
            models.Add("5008");
            models.Add("iOn");
            manufacturers.Add("Peugeot", models);

            models = new List<string>();
            models.Add("Panda");
            models.Add("Punto");
            models.Add("500");
            models.Add("Idea");
            manufacturers.Add("Fiat", models);

            models = new List<string>();
            models.Add("Ka");
            models.Add("Fiesta");
            models.Add("Focus");
            models.Add("Fusion");
            manufacturers.Add("Ford", models);


            List<string> keys = new List<string>(manufacturers.Keys);
            BindingList<object[]> bindingList = new BindingList<object[]>();

            //Generate random data
            for (int i = 0; i < 1000; ++i)
            {
                //string manufacturer = GetManufacturer();
                string manufacturer = keys[random.Next(keys.Count)];
                string model = manufacturers[manufacturer][random.Next(manufacturers[manufacturer].Count)];

                object[] data = new object[]
                                    {
                                        manufacturer,
                                        model,
                                        random.Next(1, 50),
                                        (double) random.Next(10000, 25000),
                                        DateTime.Now - TimeSpan.FromDays(2*365) + TimeSpan.FromDays(random.Next(4*365) + 1),
                                        random.Next(100),
                                        random.Next(1, 6)
                                    };

                bindingList.Add(data);
            }
            return bindingList;
        }

        private void grid_GroupingChanged(object sender, GridHeaderEventArgs e)
        {
            grid.Hierarchy.ButtonBehaviour = grid.Headers[0].GroupedColumns.Count > 0
                                                 ? ExpansionButtonBehaviour.HideWhenNoChildren
                                                 : ExpansionButtonBehaviour.HideAlways;
        }

        #region Nested type: Comparer

        private class Comparer : IComparer
        {
            #region IComparer Members

            public int Compare(object x, object y)
            {
                return string.Compare(x.ToString(), y.ToString());
            }

            #endregion
        }

        #endregion
    }
}