using System;
using System.Globalization;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;

namespace Dapfor.Net.Demo.Examples.Performance.MemoryConsumption
{
    [Example("Memory consumption", "Performance")]
    public partial class MemoryConsumptionDemoForm : UserControl
    {
        private readonly Timer _timer = new Timer();

        public MemoryConsumptionDemoForm()
        {
            InitializeComponent();

            lbMemoryEvolution.Text = string.Empty;
            tbAddRows.Text = "100000";
            tbRemoveRows.Text = "100000";

            lbTotalMemory.Text = string.Empty;
            lbRowsCount.Text = string.Empty;
            lbRowsCount.Text = string.Empty;

            _timer.Interval = 1000;
            _timer.Tick += OnTimerTick;
            _timer.Start();
        }

        private void OnTimerTick(object sender, EventArgs e)
        {
            lbTotalMemory.Text = string.Format("{0:n2} Mb", ((double) MemoryHelper.PrivateBytes)/(1024*1024));
            lbRowsCount.Text = grid.Rows.Count.ToString();
        }

        #region Private event handlers

        private void OnAddRows(object sender, EventArgs e)
        {
            int count;
            if (!string.IsNullOrEmpty(tbAddRows.Text) && int.TryParse(tbAddRows.Text, NumberStyles.Number, CultureInfo.InvariantCulture, out count) && count >= 0)
            {
                Random random = new Random();
                //Force garbage collection before adding items
                GC.Collect();
                long totalMemory = GC.GetTotalMemory(true);

                for (int i = 0; i < count; ++i)
                {
                    DataObject dataObject = new DataObject(random);
                    grid.Rows.Add(dataObject);
                }

                //Force garbage collection after items have been added
                GC.Collect();
                long totalMemory2 = GC.GetTotalMemory(true);


                int bytesPerRow = (int) ((totalMemory2 - totalMemory)/count);
                lbMemoryEvolution.Text = string.Format("Added {0} rows. Consumption: {1:n2} Mb, {2} bytes/row",
                                                       count,
                                                       ((double) (totalMemory2 - totalMemory))/(1024*1024),
                                                       bytesPerRow);
            }
        }

        private void OnRemoveRows(object sender, EventArgs e)
        {
            int count;
            if (!string.IsNullOrEmpty(tbRemoveRows.Text) && int.TryParse(tbRemoveRows.Text, NumberStyles.Number, CultureInfo.InvariantCulture, out count) && count >= 0)
            {
                count = Math.Min(count, grid.Rows.Count);
                if (count > 0)
                {
                    //Force garbage collection before adding items
                    GC.Collect();
                    long totalMemory = GC.GetTotalMemory(true);

                    for (int i = 0; i < count; ++i)
                    {
                        grid.Rows.RemoveAt(grid.Rows.Count - 1);
                    }


                    //Force garbage collection after items have been added
                    GC.Collect();
                    long totalMemory2 = GC.GetTotalMemory(true);


                    int bytesPerRow = (int) ((totalMemory - totalMemory2)/count);
                    lbMemoryEvolution.Text = string.Format("Removed {0} rows. Collected: {1:n2} Mb, {2} bytes/row",
                                                           count,
                                                           ((double) (totalMemory - totalMemory2))/(1024*1024),
                                                           bytesPerRow);
                }
            }
        }

        #endregion
    }
}