using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Internal
{
    public partial class ExamplesList : UserControl
    {
        private ExampleHolder _holder;

        public ExamplesList()
        {
            InitializeComponent();


            grid.PaintRow += OnPaintRow;
            grid.PaintCell += OnPaintCell;
            grid.CellEnter += OnCellEnter;
            grid.CellLeave += OnCellLeave;
            grid.FocusedRowChanged += OnFocusedRowChanged;
            grid.Selection.MultipleEnabled = false;

            //Set headers
            grid.Headers.Add(new Header());
            grid.Headers[0].Add(new Column("0"));
            grid.Headers[0]["0"].SortDirection = SortDirection.Ascending;
            grid.Headers[0].Visible = false;
            grid.Headers[0]["0"].Resizable = false;
            grid.Headers[0].StretchMode = ColumnStretchMode.All;
            
            
            //Look for all examples, defined in the application and register them
            Dictionary<string, Row> groups = new Dictionary<string, Row>();
            foreach (Type type in typeof (MainForm).Assembly.GetTypes())
            {
                if (type.IsSubclassOf(typeof (Control)))
                {
                    ExampleAttribute[] attributes = (ExampleAttribute[]) type.GetCustomAttributes(typeof (ExampleAttribute), false);
                    if (attributes.Length > 0)
                    {
                        Row groupRow;
                        if (!groups.TryGetValue(attributes[0].Group, out groupRow))
                        {
                            groupRow = grid.Rows.Add(new string[] {attributes[0].Group});
                            groups.Add(attributes[0].Group, groupRow);
                        }

                        groupRow.Add(new string[] {attributes[0].Name}).Tag = new ExampleInfo(attributes[0], type);
                        groupRow.Expanded = true;
                    }
                }
            }
        }

        private void OnCellLeave(object sender, GridCellEventArgs e)
        {
            e.Cell.Invalidate();
        }

        private void OnPaintRow(object sender, PaintRowEventArgs e)
        {
            switch (e.Row.Level)
            {
                case 0:
                    e.Appearance.BackColor = Color.FromArgb(217, 235, 254);
                    e.Appearance.GradientEnabled = true;
                    e.Appearance.GradientDirection = GradientDirection.Vertical;
                    e.Appearance.GradientEndBackColor = Color.FromArgb(179, 209, 238);
                    break;

                case 1:
                    e.Appearance.BackColor = Color.FromArgb(254, 250, 241);
                    e.Appearance.GradientEnabled = true;
                    e.Appearance.GradientDirection = GradientDirection.Vertical;
                    e.Appearance.GradientEndBackColor = Color.FromArgb(252, 237, 210);
                    break;
            }
        }

        private void OnPaintCell(object sender, PaintCellEventArgs e)
        {
            if(e.Cell.Row.Level > 0 && e.Cell.VisibleBounds.Contains(e.Grid.PointToClient(Cursor.Position)))
            {
                PaintHelper.AlphaBlend(e.Appearance, Color.FromArgb(150, 217, 235, 254));
            }
        }

        private void OnCellEnter(object sender, GridCellEventArgs e)
        {
            if (e.Cell != null && e.Cell.Row != null && e.Cell.Row.Level > 0)
            {
                e.Cell.Highlight(TimeSpan.FromMilliseconds(200), Color.FromArgb(150, 217, 235, 254));
            }
        }

        public ExampleHolder Holder
        {
            get { return _holder; }
            set { _holder = value; }
        }

        private void OnFocusedRowChanged(object sender, FocusedRowEventArgs e)
        {
            if (e.NewFocusedRow != null)
            {
                ExampleInfo exampleExampleInfo = (ExampleInfo)e.NewFocusedRow.Tag;
                if (exampleExampleInfo != null)
                {
                    GC.Collect();
                    
                    //Start the example
                    if ((ModifierKeys & Keys.Control) != 0)
                    {
                        FormExampleHolder form = new FormExampleHolder();
                        form.Text = string.Format("Dapfor: {0}", exampleExampleInfo.MetaData.Name);
                        form.SetControl(exampleExampleInfo.ExampleType);
                        
                        
                        form.Show(this);
                        _holder.SetControl(new DefaultDescription());
                    }
                    else
                    {
                        _holder.SetControl(exampleExampleInfo.ExampleType);    
                    }
                }
                else
                {
                    //Set default description
                    _holder.SetControl(new DefaultDescription());
                }
            }
            else if (e.Grid.Selection.Count == 0)
            {
                //Set default description
                _holder.SetControl(new DefaultDescription());
            }
        }
    }
}